<?php

namespace Lc\CaracoleBundle\Field\Filter;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Dto\FieldDto;
use Lc\ShopBundle\Context\TreeInterface;
use Lc\SovBundle\Field\Filter\AssociationFilter;
use Lc\SovBundle\Field\Filter\FilterTrait;
use Lc\SovBundle\Repository\RepositoryQueryInterface;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\FormBuilderInterface;

/**
 * @author La clic ! <contact@laclic.fr>
 */
class ProductCategoriesFilter extends AssociationFilter
{
    public function buildProperty(FormBuilderInterface $builder, FieldDto $fieldDto, $options = array())
    {
        $targetEntity = $options['entity_dto']->getPropertyMetadata($fieldDto->getProperty())->get('targetEntity');

        $builder->add(
            $fieldDto->getProperty(),
            EntityType::class,
            array(
                'class' => $targetEntity,
                'placeholder' => '--',
                'choices' => $fieldDto->getFormTypeOption('choices'),
                'required' => false,
                'attr' => array(
                    'class' => 'select2 input-sm',
                    'form' => 'filters-form',
                ),
                'choice_label' => function ($category) {
                    $isOffline = '';
                    if ($category->getStatus() != 1) {
                        $isOffline = " [Hors ligne]";
                    }
                    $section = ' ['.$category->getSection()->getTitle().']' ;;
                    return $category . $section. $isOffline;

                }
            )
        );
    }


    public function applyFilter(RepositoryQueryInterface $repositoryQuery, FieldDto $fieldDto, $filteredValue = null)
    {
        $fieldProperty = $this->getFieldProperty($fieldDto);
        if ($filteredValue !== null) {
            if ($filteredValue->getParent() == null) {
                $param = array_merge(array($filteredValue), $filteredValue->getChildrens()->toArray());
            } else {
                $param = array($filteredValue);
            }
            $repositoryQuery->filterByProductCategoryArray($param);

        }
    }

}
